package de.lmu.subspace.algorithm;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.IOException;
import java.io.Serializable;
import java.text.Format;
import java.text.NumberFormat;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.Arrays;

import Jama.*;


/**
 * Fast Interaction-based Subspace Clustering (FISC) for High Dimensional Data
 * 
 * @author Junming Shao
 * @param None
 * Created on 16. Feb. 2010, 
 */

public class FISC{
    public int num;
    public int dim;
    public String fname;
    public ArrayList src;
    public ArrayList ss;
    public ArrayList subs;
    public double[][] nsrc;
    public double[][] ssrc;
    public double[] std;
    double[][] fdata; 
    String[][] sdata; 
    public double thr; 
    public int test;
    public double order;
    /**
     * Construct function
     */	
	public FISC(String fn, double v){
		fname = fn;
		thr = v;	
		test  = 0;
	}
	
    /**
     * Load the database from a given filename
     */
    public void loadData(){
    	src = new ArrayList();    
        try {
        	File mFile = new File(fname);
        	FileReader fr = new FileReader(mFile);
        	BufferedReader br = new BufferedReader(fr);
        	String line;
        	
        	while((line=br.readLine())!=null){
        		line = line.trim();        		
        		String[] str = line.split("\\s+");
        		//System.out.println(num);
        		dim = str.length-1;
        		ArrayList temp = new ArrayList(dim);    		
        		for(int i=0;i<dim;i++){
        			temp.add(Double.parseDouble(str[i]));
        		}
    			src.add(temp);
    			num = num + 1;
        	}
        } catch (IOException ex) {
            ex.printStackTrace();
        }
    }	
	
    /**
     * Normalize the dataset
     * @param ArrayList srcDdata 
     * @return ArrayList normData
     */
    public void norm(){
    	
       int n = src.size();
       int d = ((ArrayList)src.get(0)).size();
       double [] max = new double[d];
       double [] min = new double[d];
       
       nsrc = new double[num][dim];
   	   ssrc = new double[num][dim];

	
       //Initialize
       for(int i=0;i<d;i++){
    	   max[i]= -Double.MAX_VALUE;
    	   min[i]= Double.MAX_VALUE;
       }
       
       //Find the maximum value for each variance
       for(int j=0;j<d;j++){
    	  for(int i=0;i<n;i++){
    		  double temp = (Double)((ArrayList)src.get(i)).get(j);
    		  if(temp > max[j]){
    			  max[j]= temp;
    		  }    		  
    		  if(temp < min[j]){
    			  min[j]= temp;
    		  } 
    	  }
       }
       //Normalize the data set
       for(int i=0;i<n;i++){
     	  for(int j=0;j<d;j++){
     		double temp = (Double)((ArrayList)src.get(i)).get(j);
     		ssrc[i][j] = temp;
     		nsrc[i][j] = (temp-min[j])/(max[j]-min[j]); 
     	  }
        }
       saveData(ssrc, "initial.txt"); 
    } 
    
    /**
     * Normalize the dataset
     * @param ArrayList srcDdata 
     * @return ArrayList normData
     */
    public void stdCompute(){
    	
       std = new double[dim];
       double[] mean = new double[dim];
       for(int i=0;i<dim;i++){
    	   for(int j=0;j<num;j++){
    		   mean[i] += nsrc[j][i];
    	   }
    	   mean[i] /= num;
       }
       
       for(int i=0;i<dim;i++){
    	   for(int j=0;j<num;j++){
    		   std[i] += (nsrc[j][i]-mean[i])*(nsrc[j][i]-mean[i]);
    	   }
    	   std[i] = Math.sqrt(std[i]/num);
    	   System.out.println("std:"+ std[i] );
       }
    } 

    /**
     * save the clustering result
     * @param ArrayList data (result)
     * @param String fn (filename)
     * @return none
     */
    public void saveData(ArrayList data, String fn){
        
    	int lens = data.size();
        try{
			FileOutputStream fout = new FileOutputStream(new File(fn));     
			for(int i=0;i<lens;i++){   
				ArrayList al = (ArrayList)data.get(i);
				for(int j=0; j<al.size();j++){   
					fout.write(((Double)(al.get(j))+"\t").getBytes());   
				}   
			fout.write(("\r\n").getBytes());   
			}
			fout.close();
        } catch (IOException ex) {
            ex.printStackTrace();
        }    		
    }  
    /**
     * save the clustering result
     * @param ArrayList data (result)
     * @param String fn (filename)
     * @return none
     */
    public void saveIntData(ArrayList data, String fn){
        
    	int lens = data.size();
        try{
			FileOutputStream fout = new FileOutputStream(new File(fn));     
			for(int i=0;i<lens;i++){   
				ArrayList al = (ArrayList)data.get(i);
				for(int j=0; j<al.size();j++){   
					fout.write(((Integer)(al.get(j))+"\t").getBytes());   
				}   
			fout.write(("\r\n").getBytes());   
			}
			fout.close();
        } catch (IOException ex) {
            ex.printStackTrace();
        }    		
    }   

    /**
     * save the clustering result
     * @param ArrayList data (result)
     * @param String fn (filename)
     * @return none
     */
    public void saveCluster(ArrayList data, String s, String fn){
        
    	int lens = data.size();
        try{
			FileOutputStream fout = new FileOutputStream(new File(fn));  
			fout.write((s).getBytes());
			
			for(int i=0;i<lens;i++){   
				int al = (Integer)data.get(i); 
				for (int d=0;d<dim;d++){
					fout.write(((Double)ssrc[al][d]+"\t").getBytes()); 
				}
				fout.write(("\r\n").getBytes()); 
			}
			fout.close();
        } catch (IOException ex) {
            ex.printStackTrace();
        }    		
    }
    
    /**
     * save the clustering result
     * @param ArrayList data (result)
     * @param String fn (filename)
     * @return none
     */
    public void saveClusterID(ArrayList data, String fn){
        
    	int lens = data.size();
        try{
			FileOutputStream fout = new FileOutputStream(new File(fn));  
			
			for(int i=0;i<lens;i++){   
				int al = (Integer)data.get(i); 
				fout.write((al+"\t").getBytes()); 
			}
			
			fout.close();
        } catch (IOException ex) {
            ex.printStackTrace();
        }    		
    }
    
    /**
     * save the clustering result
     * @param double[][] data (result)
     * @param String fn (filename)
     * @return none
     */    
    public void saveData(double[][] data, String fn){
        
        try{
			FileOutputStream fout = new FileOutputStream(new File(fn));     
			for(int i=0;i<data.length;i++){   
				for(int j=0; j<data[0].length;j++){   
					fout.write((data[i][j]+"\t").getBytes());   
				}   
			fout.write(("\r\n").getBytes());   
			}
			fout.close();
        } catch (IOException ex) {
            ex.printStackTrace();
        }    		
    }    

    public void save1d(double[] data, String fn){
        
        try{
			FileOutputStream fout = new FileOutputStream(new File(fn));     
			for(int i=0;i<data.length;i++){   
				fout.write((data[i]+"\t").getBytes());   
				fout.write(("\r\n").getBytes());   
			}
			fout.close();
        } catch (IOException ex) {
            ex.printStackTrace();
        }    		
    }   
    /**
     * Compute the Euclidean distance between two feature vectors
     * @param double[] x , double[] y
     * @return double dist
     */    
    public double Euclidean(double[] x, double[] y){
    	
    	double val = 0.0;
    	for(int i=0;i<x.length;i++){
    		val = val + (x[i]-y[i])*(x[i]-y[i]);
    	}   
    	double dist = Math.sqrt(val);
    	//System.out.println("Test: " + dist);
    	return dist;
    }   
    
    /**
     * Compute the Mahalanobis distance between two feature vectors
     * @param double[] x , double[] y, double[][] conv
     * @return double dist
     */    
    public double Mahalanobis(double[] x, double[] y, double[][] conv){
    	
    	double[] xy = new double[dim];
    	for(int i=0;i<dim;i++){
    		xy[i] = y[i]-x[i];
    	}   
    	
//    	//normalize the inverse covariance
//    	double sum = 0.0;
//    	for(int i=0;i<dim;i++){
//    		for(int j=0;j<dim;j++){
//    			sum+=conv[i][j];
//    		}
//    	}
//    	
//    	for(int i=0;i<dim;i++){
//    		for(int j=0;j<dim;j++){
//    			conv[i][j]/=sum;
//    		}
//    	}
//    	
    	double[] temp = new double[dim];  	
    	for(int i=0;i<dim;i++){
    		double val = 0.0;
    		for(int j=0;j<dim;j++){
    			val = val + xy[i]*conv[j][i];
    		}
    		temp[i] = val;
    	}
    	
    	double dist = 0.0;
    	for(int i=0;i<dim;i++){
    		dist = dist + temp[i]*xy[i];
    	}
    	
//    	//System.out.println("======");
//    	double dist = 0.0;
//    	for(int i=0;i<dim;i++){
//    		dist = dist + conv[i][i]*xy[i]*xy[i];
//    		//System.out.println(conv[i][i]);
//    	}    	
    	
    	dist = Math.sqrt(Math.abs(dist));
    	//System.out.println(dist);
    	return dist;
    }  
    
    /**
     * Search the eplison-range Neighborhood objects
     * @param double e , double p, double[][] data
     * @return ArrayList eplison-NN
     */    
    public ArrayList<Integer> eNN_Norm(double e, double[] p, double[][] data){
    	
    	ArrayList<Integer> neigh = new ArrayList<Integer>();
    	ArrayList<Integer> NN = new ArrayList<Integer>();
    	for(int i=0;i<num;i++){
    		if(Euclidean(p,data[i]) < e){
    			NN.add(i);
    		}
    	}
    	return NN;
    }
    
    /**
     * Search the eplison-range Neighborhood objects
     * @param double e , double p, double[][] data
     * @return ArrayList eplison-NN
     */    
    public ArrayList<Integer> eNNSearch(double e, double[] p, double[][] data){
    	
    	ArrayList<Integer> NN = new ArrayList<Integer>();
    	for(int i=0;i<num;i++){
    		double dist = 0.0;
    		int n = 0;
    		for(int d=0;d<dim;d++){
    			double dif = Math.abs(data[i][d]-p[d]);
    			//System.out.println("dif: " + dif);
    			if(dif<std[d]/5){
    				dist = dist + dif*dif;
    				n++;
    			}    			
    		}
    		if(n>0){
    			dist = Math.sqrt(dist/n);
    			//System.out.println("dist:" + dist);
    		}else{
    			dist = Double.MAX_VALUE;
    		}
    		
    		
    		
    		if(dist< e){
    			NN.add(i);
    		}
    	}
    	//System.out.println(NN.size());
    	return NN;    	
    }   
    
    /**
     * Search the eplison-range Neighborhood objects
     * @param double e , double p, double[][] data
     * @return ArrayList eplison-NN
     */    
    public ArrayList<Integer> eNNReN(double e, double[] p, double[][] data){
    	
    	ArrayList<Integer> neigh = new ArrayList<Integer>();
    	ArrayList<Integer> NN = new ArrayList<Integer>();
    	for(int i=0;i<num;i++){
    		if(Euclidean(p,data[i]) < e){
    			NN.add(i);
    		}
    	}	
    	
    
    	 
    		if(NN.size()>1){
		    	int len = NN.size();
		    	double[][] selData = new double[dim][len];
		    	double[] mean = new double[dim];
		    	
		    	for(int i=0;i<dim;i++){
		    		for(int j=0;j<len;j++){
		    			selData[i][j] = data[NN.get(j)][i];
		    		}
		    	}
		    	
		    	
				//Calculate the mean    	
		    	for(int i=0;i<dim;i++){
		    		mean[i] = 0.0;
		    		for(int j=0;j<len;j++){
		    			mean[i] += selData[i][j];
		    		}
		    		mean[i]/=len;
		    	}
			
				// Calculate the covariance
				double[][] cov = new double[dim][dim]; 
				for(int i=0;i<dim;i++){
					for(int j=0;j<dim;j++){
						double temp = 0.0;
						for(int k=0;k<len;k++){
							temp += (selData[i][k]-mean[i])*(selData[j][k]-mean[j]);
						}
						cov[i][j] = temp/(len-1);
					}
				}
				
				//Calculate the inverse covariance
				Matrix mt = new Matrix(cov);			
		        EigenvalueDecomposition eig = new EigenvalueDecomposition(mt);   
		        Matrix values = eig.getD(); 
		        Matrix vector = eig.getV();
		        double[][] theta = values.getArrayCopy();						        
				
		        // Normalize the Diagonal Elements to [0 1]
		       	double sum = 0.0;
				for(int d=0;d<dim;d++){
					if(theta[d][d]<0.0){
						//System.out.println(theta[d][d]);
						theta[d][d] = -theta[d][d];
					}
					sum = sum + theta[d][d];
				}
				
			
				
				Matrix mconv = new Matrix(theta);
				Matrix incov = vector.times(mconv).times(vector.transpose());
				double[][] insconv = incov.getArrayCopy();	

				// Mahalanobis distance
		    	for(int i=0;i<num;i++){
//		    		if(i==3 && Euclidean(p,data[i])!=0){
//		    		System.out.println("Ma==" +Mahalanobis(p,data[i],insconv)/Euclidean(p,data[i]));
//		    		//System.out.println("Ed==" +Euclidean(p,data[i]));
//		    		}
		    		if(Mahalanobis(p,data[i],insconv) < e){
		    			neigh.add(i);
		    		}
		    	}	
		    	return neigh;
	    	}else
	    		return NN;
	    	
	    	
    }   
    
    
    /**
     * Search the eplison-range Neighborhood objects
     * @param double e , double p, double[][] data
     * @return ArrayList eplison-NN
     */    
    public ArrayList<Integer> eNN(double e, double[] p, double[][] data){
    	
    	ArrayList<Integer> neigh = new ArrayList<Integer>();
    	ArrayList<Integer> NN = new ArrayList<Integer>();

    	for(int i=0;i<num;i++){
    		double tmp = Euclidean(p,data[i]);
    		if(tmp < e){
    			NN.add(i);
    		}
    	}	
    	 
    		if(NN.size()>1){
		    	int len = NN.size();
		    	double[][] selData = new double[dim][len];
		    	double[] mean = new double[dim];
		    	double[] median = new double[dim];
		    	
		    	for(int i=0;i<dim;i++){
		    		for(int j=0;j<len;j++){
		    			selData[i][j] = data[NN.get(j)][i];
		    		}
		    	}
		    	
		    	for(int d=0;d<dim;d++){
		    		Arrays.sort(selData[d]);
		    		median[d] = selData[d][(int)(len/2)];
		    	}
		    	//System.out.println(median[0]);
		    	
		    	
				// Calculate the covariance
				double[][] cov = new double[dim][dim]; 
				for(int i=0;i<dim;i++){
					for(int j=0;j<dim;j++){
						double temp = 0.0;
						for(int k=0;k<len;k++){
							temp += (selData[i][k]-median[i])*(selData[j][k]-median[j]);
						}
						cov[i][j] = temp/(len-1);
					}
				}
		    	
    
		    	
//				//Calculate the mean    	
//		    	for(int i=0;i<dim;i++){
//		    		mean[i] = 0.0;
//		    		for(int j=0;j<len;j++){
//		    			mean[i] += selData[i][j];
//		    		}
//		    		mean[i]/=len;
//		    	}
//		    	//System.out.println(mean[0]);
//		    	//System.out.println("======");		    	
//			
//				// Calculate the covariance
//				double[][] cov = new double[dim][dim]; 
//				for(int i=0;i<dim;i++){
//					for(int j=0;j<dim;j++){
//						double temp = 0.0;
//						for(int k=0;k<len;k++){
//							temp += (selData[i][k]-mean[i])*(selData[j][k]-mean[j]);
//						}
//						cov[i][j] = temp/(len-1);
//					}
//				}
				
				//Calculate the inverse covariance
				Matrix mt = new Matrix(cov);			
		        EigenvalueDecomposition eig = new EigenvalueDecomposition(mt);   
		        Matrix values = eig.getD(); 
		        Matrix vector = eig.getV();
		        double[][] theta = values.getArrayCopy();						        
				
		        // Normalize the Diagonal Elements to [0 1]
		       	//double sum = 0.0;
		       	double maxv = -Double.MAX_VALUE;
				for(int d=0;d<dim;d++){
					if(theta[d][d]<0.0){
						//System.out.println(theta[d][d]);
						//theta[d][d] = -theta[d][d];
						theta[d][d] = 0.0;
					}
					//sum = sum + theta[d][d];
					if(theta[d][d]>maxv){
						maxv = theta[d][d];
					}
				}
				

				for(int d=0;d<dim;d++){	
		
					if(theta[d][d]!=0){
						theta[d][d] = maxv/(theta[d][d]);
					}
					// for correction for too large value due to 0 element or extreme small/large elements
					if(theta[d][d]>2){
						theta[d][d] = 2;
					}
				}
				

				
				
				Matrix mconv = new Matrix(theta);
				Matrix incov = vector.times(mconv).times(vector.transpose());
				double[][] insconv = incov.getArrayCopy();	

				// Mahalanobis distance
		    	for(int i=0;i<NN.size();i++){

//		    		System.out.println("Ma==" +Mahalanobis(p,data[NN.get(i)],insconv));
//		    		System.out.println("Ed==" +Euclidean(p,data[NN.get(i)]));
//		    		System.out.println("============");
		    		
		    		if(Mahalanobis(p,data[NN.get(i)],insconv) < e){
		    			neigh.add(NN.get(i));
		    		}
		    	}	
		    	//System.out.println(neigh.size()-NN.size());
		    	return neigh;
	    	}else
	    		return NN;
	    	
	    	
    }    

    /**
     * Find the local orientation
     * @param ArrayList enn, double[][] data
     * @return ArrayList eigenvector & eigenvalue
     */       
    public ArrayList<double[][]> findLocalOrientation(ArrayList<Integer> enn, double[][] data){
    	
    	ArrayList<double[][]> al = new ArrayList<double[][]>();
    	int len = enn.size();
    	//double[][] conv = new double[len][len];
    	double[][] selData = new double[dim][len];
    	double[] mean = new double[dim];
    	
    	for(int i=0;i<dim;i++){
    		for(int j=0;j<len;j++){
    			selData[i][j] = data[enn.get(j)][i];
    		}
    	}
    	
		//Calculate the mean    	
    	for(int i=0;i<dim;i++){
    		mean[i]=0.0;
    		for(int j=0;j<len;j++){
    			mean[i] += selData[i][j];
    		}
    		mean[i]/=len;
    	}
    	
		
		// Calculate the covariance
		double[][] cov = new double[dim][dim]; 
		for(int i=0;i<dim;i++){
			for(int j=0;j<dim;j++){
				double temp = 0.0;
				for(int k=0;k<len;k++){
					temp += (selData[i][k]-mean[i])*(selData[j][k]-mean[j]);
				}
				cov[i][j] = temp/(len-1);
			}
		}
		
		Matrix mt = new Matrix(cov);
        EigenvalueDecomposition eig = new EigenvalueDecomposition(mt);   
        Matrix vectors = eig.getV();
        Matrix values = eig.getD();  
			
        double[][] vectorArray = vectors.getArrayCopy();
        double[][] valueArray = values.getArrayCopy();
        
        //saveData(vectorArray, "vector.txt");
        //saveData(valueArray, "value.txt");
        
        al.add(vectorArray);
        al.add(valueArray);    
        
    	return al;
    }
    
    /**
     * Object Mutual Interaction
     * @param ArrayList enn, double[][] data
     * @return ArrayList eigenvector & eigenvalue
     */       
    public void CoInteraction(ArrayList<Integer> enn, int idx, double[][] data, ArrayList<double[][]> orientation){    

		double[][] vector = orientation.get(0);
		double[][] weight = orientation.get(1);
		int len = enn.size();
		
		double sum = 0.0;
		for(int d=0;d<dim;d++){
			
			if(weight[d][d]<=0){
				weight[d][d] = 0.0;
			}
			sum +=weight[d][d];
		}
		for(int d=0;d<dim;d++){
			if(sum!=0){
				weight[d][d] = weight[d][d]/sum;
			}
		}
		
		double[] coupling = new double[dim];
    	for(int i=0;i<len;i++){
    		double[] diff = new double[dim];
    		
    		/*== Project the objects mutual interaction (difference) into the new space by PCA==*/
    		
    		// 1. Pair-wise difference in original space: vector difference
    		for(int d=0;d<dim;d++){
    			diff[d] = data[enn.get(i)][d] - data[idx][d];
    		}

    		// 2. Calculate the inner product between vector difference and each of the new space axis
    		// projection = (a.b).b // where b is the unit vector
    		
    		for(int k=0;k<dim;k++){
    			//inner product
    			double inp =0.0;
	    		for(int d=0;d<dim;d++){
	    			inp += diff[d]*vector[d][k];
	    		}
	    		
	    		// the projection of vector difference in each axis in the new space
	    		double[] proj = new double[dim];
	    		for(int d=0;d<dim;d++){
	    			proj[d] = inp*vector[d][k];
	    		}

	    		for(int d=0;d<dim;d++){
    				coupling[d] = coupling[d] + weight[k][k]* Math.sin(proj[d]);
    				//coupling[d] = coupling[d] + (1.0/dim)* Math.sin(proj[d]);
	    		}	    		
    		}		
    	}
    	
		for(int d=0;d<dim;d++){
			nsrc[idx][d] += coupling[d]/len;
			if(len>0){
				order += Math.abs(coupling[d]/len);
			}
		}
		
    }
    
    
    public double angle(double[] v1, double[] v2){
    	
    	double ip = 0.0;
    	double nv1 = 0.0;
    	double nv2 = 0.0;
    	for(int i=0;i<dim;i++){
    		ip += v1[i]*v2[i];
    		nv1 += v1[i]*v1[i];
    		nv2 += v2[i]*v2[i];
    	}
    	
//    	double agl = 0.0;
//    	if(nv1>0 && nv2 >0){
//    		double temp = Math.abs(ip/Math.sqrt(nv1*nv2));
//    		if(temp<=1.0){
//    			agl = Math.acos(temp);
//    		}
//
//    	
////    	System.out.println("...ip==" +ip);
////    	System.out.println("...v1==" +nv1 + " nv2 = " + nv2);
////    		if(Math.abs(ip/(Math.sqrt(nv1*nv2)))>1){
////    	    	System.out.println("...ip==" +ip);
////    	    	System.out.println("...v1==" +nv1 + " nv2 = " + nv2);
//////    	if(agl<0){
//////    		agl = -agl;
//////    	}
////    	System.out.println("===agl== " + agl);}
//    	}
//    	return agl;
    	
    	
    	double proj = 0.0;
    	if(nv1>0.0){
    		proj = Math.abs(ip/Math.sqrt(nv1));
    	}
    	return proj;
    }
    
    
    public double Nom(double[] v1){
    	double s = 0.0;
    	for(int i=0; i<v1.length;i++){
    		s += v1[i]*v1[i];
    	}
    	return Math.sqrt(s);
    }
    
    
    /**
     * Object Mutual Interaction
     * @param ArrayList enn, double[][] data
     * @return ArrayList eigenvector & eigenvalue
     */       
    public void AngInteraction(ArrayList<Integer> enn, int idx, double[][] data){    

		int len = enn.size();
		double[] ang = new double[len];
		double[] coupling = new double[dim];
		for(int i=0;i<len;i++){
			double[] diff1 = new double[dim];
			ang[i] = 0.0;
    		for(int d=0;d<dim;d++){
    			diff1[d] = data[enn.get(i)][d] - data[idx][d];
    		}
			for(int j=0;j<len;j++){
				if(j!=i){
					double[] diff2 = new double[dim];
		    		for(int d=0;d<dim;d++){
		    			diff2[d] = data[enn.get(j)][d] - data[idx][d];
		    		}
		    		double agl = angle(diff1, diff2);
		    		ang[i] = ang[i] + agl; 
				}
			}
			//System.out.println("==== " + ang[i]);
			//double nm = Nom(diff1);
			for(int d=0;d<dim;d++){	
				if(ang[i]!=0)
					//coupling[d] = coupling[d] + (1.0/ang[i])* Math.sin(diff1[d]);
					coupling[d] = coupling[d] + (ang[i])* Math.sin(diff1[d]);
    		}	
		}
		
		double sumanl = 0.0;
//		for(int i=0;i<len;i++){
//			if(ang[i]!=0)
//				sumanl +=(1.0/ang[i]); 
//		}
		for(int i=0;i<len;i++){
				sumanl += ang[i]; 
		}		
		
	    if(sumanl!=0){
			for(int d=0;d<dim;d++){
				nsrc[idx][d] += coupling[d]/sumanl;
			}
	    }

    }
    
    
    public double Projection(double[] v , double[] base){
    	
    	double proj = 0.0;
    	double in = 0.0;
    	for(int i=0;i<v.length;i++){
    		in += v[i]*base[i];
    	}
    	
    	double nv = Nom(v);
    	if(nv!=0){
    		proj = Math.abs(in/nv);
    	}
    	
    	return proj;
    }
    /**
     * Object Mutual Interaction
     * @param ArrayList enn, double[][] data
     * @return ArrayList eigenvector & eigenvalue
     */       
    public void MoveDirectionInteraction(ArrayList<Integer> enn, int idx, double[][] cur, double[][] pre){    

		int len = enn.size();
		double[] proj = new double[len];
		double[] coupling = new double[dim];
		double[] moveDirection = new double[dim];
		
		double[] preCenter = new double[dim];
		double[] curCenter = new double[dim];
		
		for(int i=0;i<len;i++){
			for(int d=0;d<dim;d++){
				preCenter[d] += pre[i][d];
				curCenter[d] += cur[i][d];
				                        
			}
		}
	
		for(int d=0;d<dim;d++){
			moveDirection[d] = (curCenter[d] - preCenter[d])/len;
		}
		double nm = Nom(moveDirection);
		if(nm!=0){
			for(int d=0;d<dim;d++){
				moveDirection[d] = moveDirection[d]/nm;
			}
		
			//System.out.println("norm of direction " +Nom(moveDirection));
			for(int i=0;i<len;i++){
				double[] diff = new double[dim];
	    		for(int d=0;d<dim;d++){
	    			diff[d] = cur[enn.get(i)][d] - cur[idx][d];
	    		}
	    		
	    		proj[i] = Projection(diff, moveDirection);
				//System.out.println("***==== " + proj[i]);
	    		
    			double inp =0.0;
	    		for(int d=0;d<dim;d++){
	    			inp += diff[d]* moveDirection[d];
	    		}

	    		// the projection of vector difference in each axis in the new space
	    		double[] pro = new double[dim];
	    		for(int d=0;d<dim;d++){
	    			pro[d] = inp*moveDirection[d];
	    		}
	    		
				for(int d=0;d<dim;d++){	
					if(proj[i]!=0)
						//coupling[d] = coupling[d] + (1.0/ang[i])* Math.sin(diff1[d]);
						//coupling[d] = coupling[d] + (proj[i])* Math.sin(diff[d]);
						coupling[d] = coupling[d] + (proj[i])* Math.sin(pro[d]);
	    		}	
			}		
		}else{
			
			for(int i=0;i<len;i++){
				double[] diff = new double[dim];
	    		for(int d=0;d<dim;d++){
	    			diff[d] = cur[enn.get(i)][d] - cur[idx][d];
	    		}
	    		proj[i] = 1;
				for(int d=0;d<dim;d++){	
					coupling[d] = coupling[d] + (proj[i])* Math.sin(diff[d]);
	    		}
			}
		}
		
		
		double sumanl = 0.0;
		for(int i=0;i<len;i++){
				sumanl += proj[i]; 
		}		
		
	    if(sumanl!=0){
			for(int d=0;d<dim;d++){
				nsrc[idx][d] += coupling[d]/sumanl;
			}
	    }

    }
    
    
    public void InitInteraction(ArrayList<Integer> enn, int idx, double[][] cur){    

		int len = enn.size();
		double[] coupling = new double[dim];
		for(int i=0;i<len;i++){
			double[] diff = new double[dim];
    		for(int d=0;d<dim;d++){
    			diff[d] = cur[enn.get(i)][d] - cur[idx][d];
    		}
			for(int d=0;d<dim;d++){	
					coupling[d] = coupling[d] + Math.sin(diff[d]);	
			}
		}
	
		for(int d=0;d<dim;d++){
			nsrc[idx][d] += coupling[d]/(len);
		}
    }
    
    public void interaction(){
    	
    	long   start2   =   System.currentTimeMillis(); 
    	System.out.println("Number of Objects:" + num + "; Dimensionality: " + dim);
    	boolean loop = true;
    	int loopNum = 0;

    	saveData(nsrc, "synC0.txt"); // Record the initial state of objects
    	
    	double[][] cur = new double[num][dim];
    	double[] r = new double[30];
    	// Start to cluster by mutual interaction according to the local structure of objects
    	System.out.println("Start to clustering by local mutual interaction.");
    	while(loop){
        	// Keep the current values to variable "cur"

        	for(int i=0;i<num;i++){   		
        		for(int j=0;j<dim;j++){
        			cur[i][j] = nsrc[i][j];
        		}	
        	}

    		//System.out.println("Loop:" + loopNum);
    		loopNum = loopNum + 1;
    		
    		order = 0.0;
	    	for(int i=0;i<num;i++){	    		
    			//Search the eplison-neighborhood objects of one object 
    			ArrayList<Integer> enn = eNN(thr, cur[i], cur); 
    			//ArrayList<Integer> enn = eNNSearch(thr, cur[i], cur);
    			//ArrayList<Integer> enn = eNN_Norm(thr, cur[i], cur); 
    			//System.out.println("new: " + enn.size());
    			// find the local orientation around the current object
    			
    			if(enn.size()>1){
    				ArrayList<double[][]> orientation = findLocalOrientation(enn, cur);				
    				CoInteraction(enn,i, cur, orientation);
    			}   			
	    	}
	    	order = order/(num*dim);
	    	//r[loopNum-1] = order;
			//saveData(nsrc, "synC"+String.valueOf(loopNum)+".txt");

			if( order<1e-5 ||loopNum >30 ){
				loop = false;
			}    
			System.out.println("Loop: " + loopNum + " Order = " + order);
    	}
    	//saveData(nsrc, "syc.txt");
    	//save1d(r, "r.txt");
        long   end2   =   System.currentTimeMillis(); 
    	System.out.println("Find the synchronized Phases:"+Long.toString((end2-start2)/1000)+" s.");	
    	
    }
    
    /** Find the related objects in corresponding subspace
     * 
     * @param ArrayList ss,  all possible subspaces 
     * @param ArrayList al,  the all possible spaces of current object
     */
	public ArrayList DBSearch(ArrayList cur){
		
		ArrayList relDB = new ArrayList();
		int id = (Integer)cur.get(0);
		ArrayList al = (ArrayList)cur.get(1); //subspace
		ArrayList ap = (ArrayList)cur.get(2); //synchronized phases
//		for(int i=0;i<al.size();i++){
//			System.out.print(" " + (Integer)al.get(i));
//		}
//		System.out.println("");
//		for(int i=0;i<ap.size();i++){
//			System.out.print(" " + (Double)ap.get(i));
//		}
		relDB.add(al); // Subspace
		relDB.add(ap); // Phases
		ArrayList cluster = new ArrayList(); 
		cluster.add(id); // add the first object to the corresponding cluster
		
		for(int i=0;i<num;i++){
			int flag = 0;
			ArrayList each = (ArrayList)ss.get(i);
			// if this object exist any relevant dimension
			if(each.size()>1){
				ArrayList es = (ArrayList)each.get(1); // get the corresponding subspace
				ArrayList ep = (ArrayList)each.get(2); // the corresponding phases
//				System.out.println("");
//				for(int x=0;x<al.size();x++){
//					System.out.print(" " + (Integer)es.get(x));
//				}
//				System.out.println("");
//				for(int y=0;y<ap.size();y++){
//					System.out.print(" " + (Double)ep.get(y));
//				}
//				
//				System.out.println("es.size " + es.size());
//				System.out.println("al.size " + al.size());
				if(i!=id && es.size()>=al.size()){
					for(int j=0;j<al.size();j++){
						for(int k=0;k<es.size();k++){
							
							//check whether exist any object has same phase in	same subspace						
							if((Integer)es.get(k)==(Integer)al.get(j)){
								//System.out.println("loop: j = " + j + " k =" + k);
								//System.out.println("ep = " + (Double)ep.get(k) + " ap =" + (Double)ap.get(k));
								//System.out.println("ep: "+(Double)ep.get(k));
								//System.out.println("ap: "+(Double)ap.get(j));
									if(Math.abs((Double)ep.get(k)-(Double)ap.get(j))<1e-10){
										flag = flag + 1;
									}
							}
						}
						
					}	
				}
				//if the object match, add the object in the cluster
				//System.out.println("size" + al.size()+ " flag=" + flag);
				if(flag==al.size()){
					//System.out.println("sss");
					cluster.add(i);
				}
			}
		}
		relDB.add(cluster); //add the cluster
		relDB.add(cluster.size()); // add the number of the cluster
		return relDB;
	}
   
	
    /** Find the related objects in corresponding subspace
     * 
     * @param ArrayList ss,  all possible subspaces 
     * @param ArrayList al,  the all possible spaces of current object
     */
	public ArrayList SubspaceSearch(double[][] fdata){
		
		ArrayList subs = new ArrayList();
		for(int i=0;i<num;i++){
			
			double dif = 0.0;
			ArrayList eachSub = new ArrayList();
			for(int j=i+1;j<num;j++){
				
				// Find the synchronized dimension with object i
				ArrayList relDim = new ArrayList();
				for(int d=0;d<dim;d++){
					dif = fdata[i][d]-fdata[j][d];
					if(dif==0){
						relDim.add(d);
					}
				}
				
				// if there exist any synchronized dimension with object i
				if(relDim.size()>0){
					
					if(eachSub.size()==0){
						eachSub.add(relDim);
					}else{
					
						int flag = 0;
						for(int k=0;k<eachSub.size();k++){
							ArrayList al = (ArrayList)eachSub.get(k);
							int f =1;
							if(relDim.size()==al.size()){
								for(int p=0;p<relDim.size();p++){
									int ff = 0;
									if((Integer)relDim.get(p)==(Integer)al.get(p)){
										ff = 1;
									}	
									f = f*ff;
								}
							}
							// if there exist the same subspace, exist the loop
							if(f==1){
								flag = 1;
								break;
							}
						}
						
						if(flag==0){
							eachSub.add(relDim);
						}
					}			
				}	
			}
			subs.add(eachSub);
		}
		
		return subs;

	}
	
	
	public int svd(ArrayList cl, ArrayList al, double thr){
		
    	int len = cl.size();
    	//double[][] conv = new double[len][len];
    	double[][] selData = new double[dim][len];
    	double[] mean = new double[dim];
    	int nd = al.size();
    	for(int i=0;i<nd;i++){
    		for(int j=0;j<len;j++){
    			selData[i][j] = (Double)((ArrayList)src.get((Integer)cl.get(j))).get((Integer)al.get(i));
    		}
    	}
    	
		//Calculate the mean    	
    	for(int i=0;i<nd;i++){
    		mean[i]=0.0;
    		for(int j=0;j<len;j++){
    			mean[i] += selData[i][j];
    		}
    		mean[i]/=len;
    	}
		
		// Calculate the covariance
		double[][] cov = new double[nd][nd]; 
		for(int i=0;i<nd;i++){
			for(int j=0;j<nd;j++){
				double temp = 0.0;
				for(int k=0;k<len;k++){
					temp += (selData[i][k]-mean[i])*(selData[j][k]-mean[j]);
				}
				cov[i][j] = temp/(len-1);
			}
		}
		
		Matrix mt = new Matrix(cov);
        EigenvalueDecomposition eig = new EigenvalueDecomposition(mt);   
        Matrix vectors = eig.getV();
        Matrix values = eig.getD();  
			
        double[][] valueArray = values.getArrayCopy();
        int r = 0;
        double s = 0.0;
        for(int i=0;i<nd;i++){
        	s += valueArray[i][i];
        }
        for(int i=0;i<nd;i++){
        	valueArray[i][i] = valueArray[i][i]/s;
        	if(valueArray[i][i]>thr){
        		r = r + 1;
        	}
        }
        return r;
	}
	
	
	
	
	
	   /** Find the related objects in corresponding subspace
     * 
     * @param ArrayList ss,  all possible subspaces 
     * @param ArrayList al,  the all possible spaces of current object
     */
	public ArrayList ClusterSearch()  {
		
		long   start1   =   System.currentTimeMillis(); 
		
		try{
			File dir = new File("Output");
			dir.mkdir();
		}catch(Exception e){
			System.out.println("Error: Creating directory");
		}
		
		NumberFormat famatter = NumberFormat.getNumberInstance(); 
		if(dim>10)
			famatter.setMaximumFractionDigits(1);
		else{
			famatter.setMaximumFractionDigits(2);
		}
		fdata = new double[num][dim];
		for(int i=0;i<num;i++){                 
			for(int j=0;j<dim;j++){
	            String sData  = famatter.format(nsrc[i][j]);  
	            try {
					double value = famatter.parse(sData).doubleValue();
//					if(nsrc[i][j]>-1e-20 && nsrc[i][j]<0){
//						fdata[i][j]=-1e-30;
//					}
//					else if(nsrc[i][j]<1e-30 && nsrc[i][j]>0){
//						fdata[i][j]=0.0;
//					}else {
//						fdata[i][j] = value;
//					}
					
					fdata[i][j] = value;
					
				} catch (ParseException e) {
					e.printStackTrace();
				} 
			}  
		}
		long   end1   =   System.currentTimeMillis(); 
		System.out.println("Format:"+Long.toString((end1-start1)/1000)+" s.");      
		saveData(fdata, "fdata.txt");
		
		ArrayList res = new ArrayList();
		ArrayList cls = new ArrayList();
		ArrayList subs = new ArrayList();
		
		for(int i=0;i<num;i++){
			
			for(int j=i+1;j<num;j++){
				// Find the synchronized dimension with object i
				ArrayList relDim = new ArrayList();
				for(int d=0;d<dim;d++){
					if(fdata[i][d]==fdata[j][d]){
						relDim.add(d);
					}
				}
				
				// if there exist any synchronized dimension with object i
				if(relDim.size()>0){
					if(subs.size()==0){
						ArrayList temp = new ArrayList();
						temp.add(i);
						temp.add(j);
						subs.add(relDim);
						cls.add(temp);	
						
					}else{
						//check whether exist
						int flag = 0;
						for(int k=0;k<subs.size();k++){
							ArrayList al = (ArrayList)subs.get(k);
	
							
							if(relDim.size()==al.size()){
								int f =0;
								for(int p=0;p<relDim.size();p++){
									if(((Integer)relDim.get(p)==(Integer)al.get(p)) && (fdata[(Integer)(((ArrayList)cls.get(k)).get(0))][(Integer)relDim.get(p)]==fdata[j][(Integer)al.get(p)])){
										f = f + 1 ;
									}	
								}
							
							// if there exist the same subspace, exist the loop
							if(f==relDim.size()){
								flag = 1;
								ArrayList t = ((ArrayList)cls.get(k));
								int ff = 0;
								for(int x=0;x<t.size();x++){
									if(j==(Integer)t.get(x)){
										ff = 1;
										break;
									}
								}
								if(ff==0){
									((ArrayList)cls.get(k)).add(j);
								}
								//System.out.print(" " +j );
								break;
							}
						}
							if(flag==1) break; 
					}
					if(flag==0){
						
						ArrayList temp = new ArrayList();
						temp.add(i);
						temp.add(j);
						subs.add(relDim);
						cls.add(temp);	
						//System.out.print(".");
					}
					}			
				}	
			}
		}
		
		res.add(subs);
		res.add(cls);
		
		//System.out.println("Number of subspace clusters: " +subs.size());
		//System.out.println("Number of subspace clusters: " +cls.size());
		
		
 
		try{
			File[] subdir = new File[dim]; 
			for(int i=0;i<dim;i++){
				subdir[i] = new File("Output/dim".concat(String.valueOf(i+1)));
				//subdir[i].createDirectories(Paths.get("c:\\temp\\first\\abc"));
				subdir[i].mkdir();
			}
		}catch(Exception e){
			System.out.println("Error: create sub-directory");
		}
			
		try{
		  	
	    	int[] nn = new int[dim];
	    	int[] snn = new int[dim];
	    	for(int i=0;i<dim;i++){
	    		nn[i] = 0;
	    		snn[i] = 0;
	    	}
	    	int count = 0;
			for(int i=0;i<subs.size();i++){
				ArrayList al = (ArrayList)subs.get(i);
				
				if(((ArrayList)cls.get(i)).size()>0.01*num && ((ArrayList)cls.get(i)).size()>2){
					int r = svd((ArrayList)cls.get(i), al, 0.2/dim);
					if(r>0){
						nn[r-1]++;	
						snn[al.size()-1]++;
						count++;
						String s = "";
						//System.out.print("Cluster " + count+ ": ");
						//System.out.print("Synchronized Dimensions: [");
						s = s + "#================================\n";
						s = s + "#Synchronized Dimensions: [";
						for(int k=0;k<al.size();k++){
							//System.out.print((Integer)al.get(k)+" ");
							s = s + (Integer)al.get(k)+" ";
						}
						s = s + "]; \n";
						//System.out.print("];  ");
						//System.out.print("Cluster Size: " + ((ArrayList)cls.get(i)).size() + "; ");
						s = s + "#Cluster Size: " + ((ArrayList)cls.get(i)).size() + "; \n";
						//System.out.println("Dimensionality: " + r);	
						s = s + "#Dimensionality: " + r +"\n";
						s = s + "#================================\n";
//						String fn  = (("Output/dim".concat(String.valueOf(r))).concat("/cluster".concat(String.valueOf(nn[r-1])))).concat(".txt");  
//						saveCluster((ArrayList)cls.get(i),fn);
						String fn  = (("Output/dim".concat(String.valueOf(al.size()))).concat("/syncluster".concat(String.valueOf(snn[al.size()-1])))).concat(".txt");
						String fnID  = (("Output/dim".concat(String.valueOf(al.size()))).concat("/synclusterID".concat(String.valueOf(snn[al.size()-1])))).concat(".txt");
						//System.out.println(s);
						saveCluster((ArrayList)cls.get(i), s, fn);
						saveClusterID((ArrayList)cls.get(i), fnID);
					}
				}
			}
		}catch(Exception e){
			System.out.println("Error: Save clusters");
		}
		return res; 
	}
	
	
    /** Search all possible subspace
     * 
     * @param none
     */
	public void subspaceSearch(){
      
		long   start1   =   System.currentTimeMillis(); 
		NumberFormat famatter = NumberFormat.getNumberInstance();  
		famatter.setMaximumFractionDigits(2);
		fdata = new double[num][dim];
		for(int i=0;i<num;i++){                 
			for(int j=0;j<dim;j++){
	            String sData  = famatter.format(nsrc[i][j]);  
	            try {
					double value = famatter.parse(sData).doubleValue();
//					if(nsrc[i][j]>-1e-20 && nsrc[i][j]<0){
//						fdata[i][j]=-1e-30;
//					}
//					else if(nsrc[i][j]<1e-30 && nsrc[i][j]>0){
//						fdata[i][j]=0.0;
//					}else {
//						fdata[i][j] = value;
//					}
					
					fdata[i][j] = value;
					
				} catch (ParseException e) {
					e.printStackTrace();
				} 
			}  
		}
		long   end1   =   System.currentTimeMillis(); 
		System.out.println("Format:"+Long.toString((end1-start1)/1000)+" s.");      
		saveData(fdata, "fdata.txt");
	    
//		for(int i=1;i<num;i++){
//			for(int d=0;d<dim;d++){		
//			System.out.print(" " + fdata[i][d]);	
//			}
//			System.out.println("");
//		}
	    // find the distinct phase at each dimension
		long   start2   =   System.currentTimeMillis(); 
		
		ArrayList al = new ArrayList();
		ArrayList count = new ArrayList();
		for(int d=0;d<dim;d++){
			double phase = 0.0;
			ArrayList<Double> temp1 = new ArrayList<Double>();
			temp1.add(fdata[0][d]);
			ArrayList temp2 = new ArrayList();
			temp2.add(1);
			for(int i=1;i<num;i++){
				int f = 0;
				//System.out.println("fdata[" + i +"]["+d+"]" + fdata[i][d]);
				for(int j=0;j<temp1.size();j++){
					if(fdata[i][d]==(Double)temp1.get(j)){
						f = 1;
						int curr = (Integer)temp2.get(j);
						curr = curr + 1;
						temp2.set(j, curr);
					}
				}
				if(f==0){
					temp1.add(fdata[i][d]);
					//System.out.println(fdata[i][d]);
					temp2.add(1);
				}
			}
			al.add(temp1);
			count.add(temp2);
		}
	  
		saveData(al, "phases.txt");
		saveIntData(count, "count.txt");
		
		ArrayList selPhase = new ArrayList();
		for(int i=0;i<count.size();i++){
			ArrayList temp1 = (ArrayList)count.get(i);
			ArrayList temp2 = (ArrayList)al.get(i);
			ArrayList temp3 = new ArrayList();
			for(int j=0;j<temp1.size();j++){
				if((Integer)temp1.get(j)>10){
					temp3.add((Double)temp2.get(j));
				}
			}
			selPhase.add(temp3);
		}
		this.saveData(selPhase, "selPhase.txt");
		
	    long   end2   =   System.currentTimeMillis(); 
		System.out.println("Find the synchronized Phases:"+Long.toString((end2-start2)/1000)+" s.");	
		
		// Find the possible all relevant dimensions for each point
		ss = new ArrayList();
		for(int i=0;i<num;i++){
			ArrayList eachRelDim = new ArrayList();
			ArrayList relDim = new ArrayList();
			ArrayList relPhase = new ArrayList();
			for(int d=0;d<dim;d++){
				for(int k=0;k<((ArrayList)selPhase.get(d)).size();k++){
					if(fdata[i][d]==(Double)((ArrayList)selPhase.get(d)).get(k)){						
						relDim.add(d); // Relevant dimension	
						relPhase.add(fdata[i][d]); //The corresponding phase value
					}
				}
			}
			eachRelDim.add(i); //Object ID
			if(relDim.size()>0){
				eachRelDim.add(relDim); // Relevant Dimensions
				eachRelDim.add(relPhase); //The corresponding phase values
			}
			ss.add(eachRelDim);
		}	
			
		// find the related objects in corresponding subspace
		ArrayList DB = new ArrayList(); //Save the all relevant information, including subspace, phases, cluster and number of cluster
		ArrayList rsc = new ArrayList(); // Save the real subspace
		ArrayList psc = new ArrayList(); // for save each object's subspace
		for(int i=0;i<num;i++){
			// For each candidate subspace
			ArrayList cur = (ArrayList)ss.get(i); // the information for each Object, including object id, relevant dimensions and phase values 
			// If there exist any relevant dimension for the object
			if(cur.size()>1){ 
				//System.out.println("s=" + i);
				//first subspace search
				if(rsc.size()==0){
					ArrayList sc = DBSearch(cur);
					if((Integer)sc.get(sc.size()-1)>10){
						DB.add(sc);
						rsc.add((ArrayList)cur.get(1));
						psc.add((ArrayList)cur.get(2));
					}
				}else{
					
					ArrayList as = (ArrayList)cur.get(1);
					ArrayList ap = (ArrayList)cur.get(2);
					int f = 0;
					
					for(int k=0;k<rsc.size();k++){
						if(as.size()==((ArrayList)rsc.get(k)).size()){
							ArrayList rap = (ArrayList)psc.get(k);
							ArrayList rscc = (ArrayList)rsc.get(k);
							for(int p=0;p<as.size();p++){
								if((Integer)as.get(p)==(Integer)rscc.get(p)){
									if(Math.abs((Double)rap.get(p)-(Double)ap.get(p))<1e-10){
										f = f + 1;
									}
								}
							}
						}
					}
					
					// a new subspace
					if(f<as.size()){
						System.out.println("new space... as.size= " + as.size()+ "f = " + f );
						ArrayList sc = DBSearch(cur);
						if((Integer)sc.get(sc.size()-1)>10){
							//System.out.println("test....");
							DB.add(sc);
							rsc.add((ArrayList)cur.get(1));
							psc.add((ArrayList)cur.get(2));
						}
					}
				}
				
			}			
		}

		System.out.println("Totol number of objects: "+num);
		System.out.println("Totol number of subspace: "+rsc.size());		
		for(int i=0;i<DB.size();i++){
			ArrayList res = (ArrayList)DB.get(i);
			System.out.print("subspace " + i +": [");
			ArrayList subs = (ArrayList)res.get(0);
			ArrayList phs = (ArrayList)res.get(1);
			String fn = "cluster";
			for(int j=0;j<subs.size();j++){
				System.out.print(""+subs.get(j)+" ");
				fn = fn+"_"+subs.get(j);
			}
			fn = fn + "_" +i+ ".txt";
			System.out.print("]");
			//saveCluster((ArrayList)res.get(2),fn);
			System.out.print("  Synchronized Phases: [");
			for(int j=0;j<phs.size();j++){
				System.out.print(""+phs.get(j)+" ");
			}
			System.out.print("]");
			
			System.out.println(" "+(Integer)res.get(3));
		}		
	}
 
	 /**
     * main function 
     */
    public static void main(String[] args)  {
    	
    	long   start   =   System.currentTimeMillis(); 
    	
    	FISC cl = new FISC("E:\\Document\\workplace\\Subspace\\met.txt",0.5); //lineInPlane3d.txt DSNonL
    	cl.loadData();
    	cl.norm(); 
    	//cl.stdCompute();
    	cl.interaction(); 
    	//cl.subspaceSearch(); 
    	cl.ClusterSearch();

  	  	long   end   =   System.currentTimeMillis(); 
  	  	System.out.println("All running time:"+Long.toString((end-start)/1000)+" s.");      
  	  	System.out.println("Ending");	
    }		
}

