package de.lmu.subspace.algorithm;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.IOException;
import java.io.Serializable;
import java.text.Format;
import java.text.NumberFormat;
import java.text.ParseException;
import java.util.ArrayList;

import Jama.*;


/**
 * Fast Interaction-based Subspace Clustering (FISC) for High Dimensional Data
 * 
 * @author Junming Shao
 * @param None
 * Created on 16. Feb. 2010, 
 */

public class subTest{
    public int num;
    public int dim;
    public String fname;
    public ArrayList src;
    public ArrayList ss;
    public double[][] nsrc;
    double[][] fdata; 
    String[][] sdata; 
    public double thr; 
    
    /**
     * Construct function
     */	
	public subTest(String fn, double v){
		fname = fn;
		thr = v;	
	}
	
    /**
     * Load the database from a given filename
     */
    public void loadData(){
    	src = new ArrayList();    
        try {
        	File mFile = new File(fname);
        	FileReader fr = new FileReader(mFile);
        	BufferedReader br = new BufferedReader(fr);
        	String line;
        	
        	while((line=br.readLine())!=null){
        		line = line.trim();        		
        		String[] str = line.split("\\s+");
        		//System.out.println(num);
        		dim = str.length-1;
        		ArrayList temp = new ArrayList(dim);    		
        		for(int i=0;i<dim;i++){
        			temp.add(Double.parseDouble(str[i]));
        		}
    			src.add(temp);
    			num = num + 1;
        	}
        } catch (IOException ex) {
            ex.printStackTrace();
        }
    }	
	
    /**
     * Normalize the dataset
     * @param ArrayList srcDdata 
     * @return ArrayList normData
     */
    public void norm(){
    	
       int n = src.size();
       int d = ((ArrayList)src.get(0)).size();
       double [] max = new double[d];
       double [] min = new double[d];
       
       nsrc = new double[num][dim];
       
       //Initialize
       for(int i=0;i<d;i++){
    	   max[i]= -Double.MAX_VALUE;
    	   min[i]= Double.MAX_VALUE;
       }
       
       //Find the maximum value for each variance
       for(int j=0;j<d;j++){
    	  for(int i=0;i<n;i++){
    		  double temp = (Double)((ArrayList)src.get(i)).get(j);
    		  if(temp > max[j]){
    			  max[j]= temp;
    		  }    		  
    		  if(temp < min[j]){
    			  min[j]= temp;
    		  } 
    	  }
       }
       //Normalize the data set
       for(int i=0;i<n;i++){
     	  for(int j=0;j<d;j++){
     		double temp = (Double)((ArrayList)src.get(i)).get(j);
     		nsrc[i][j] = (temp-min[j])/(max[j]-min[j]); 
     	  }
        }	
    } 

    /**
     * save the clustering result
     * @param ArrayList data (result)
     * @param String fn (filename)
     * @return none
     */
    public void saveData(ArrayList data, String fn){
        
    	int lens = data.size();
        try{
			FileOutputStream fout = new FileOutputStream(new File(fn));     
			for(int i=0;i<lens;i++){   
				ArrayList al = (ArrayList)data.get(i);
				for(int j=0; j<al.size();j++){   
					fout.write(((Double)(al.get(j))+"\t").getBytes());   
				}   
			fout.write(("\r\n").getBytes());   
			}   
        } catch (IOException ex) {
            ex.printStackTrace();
        }    		
    } 
    
    public void saveStrData(ArrayList data, String fn){
        
    	int lens = data.size();
        try{
			FileOutputStream fout = new FileOutputStream(new File(fn));     
			for(int i=0;i<lens;i++){   
				ArrayList al = (ArrayList)data.get(i);
				for(int j=0; j<al.size();j++){   
					fout.write(((String)(al.get(j))+"\t").getBytes());   
				}   
			fout.write(("\r\n").getBytes());   
			}   
        } catch (IOException ex) {
            ex.printStackTrace();
        }    		
    } 
   public void saveStrrData(String[][] data, String fn){
        
    	int lens = data.length;
        try{
			FileOutputStream fout = new FileOutputStream(new File(fn));     
			for(int i=0;i<lens;i++){   
				for(int j=0; j<data[0].length;j++){   
					fout.write(((String)(data[i][j])+"\t").getBytes());   
				}   
			fout.write(("\r\n").getBytes());   
			}   
        } catch (IOException ex) {
            ex.printStackTrace();
        }    		
    }     
    /**
     * save the clustering result
     * @param ArrayList data (result)
     * @param String fn (filename)
     * @return none
     */
    public void saveIntData(ArrayList data, String fn){
        
    	int lens = data.size();
        try{
			FileOutputStream fout = new FileOutputStream(new File(fn));     
			for(int i=0;i<lens;i++){   
				ArrayList al = (ArrayList)data.get(i);
				for(int j=0; j<al.size();j++){   
					fout.write(((Integer)(al.get(j))+"\t").getBytes());   
				}   
			fout.write(("\r\n").getBytes());   
			}   
        } catch (IOException ex) {
            ex.printStackTrace();
        }    		
    }   

    /**
     * save the clustering result
     * @param ArrayList data (result)
     * @param String fn (filename)
     * @return none
     */
    public void saveCluster(ArrayList data, String fn){
        
    	int lens = data.size();
        try{
			FileOutputStream fout = new FileOutputStream(new File(fn));     
			for(int i=0;i<lens;i++){   
				int al = (Integer)data.get(i);  
					fout.write((al+"\t").getBytes());   
				}    
        } catch (IOException ex) {
            ex.printStackTrace();
        }    		
    }
    /**
     * save the clustering result
     * @param double[][] data (result)
     * @param String fn (filename)
     * @return none
     */    
    public void saveData(double[][] data, String fn){
        
        try{
			FileOutputStream fout = new FileOutputStream(new File(fn));     
			for(int i=0;i<data.length;i++){   
				for(int j=0; j<data[0].length;j++){   
					fout.write((data[i][j]+"\t").getBytes());   
				}   
			fout.write(("\r\n").getBytes());   
			}   
        } catch (IOException ex) {
            ex.printStackTrace();
        }    		
    }    

    /**
     * Compute the Euclidean distance between two feature vectors
     * @param double[] x , double[] y
     * @return double dist
     */    
    public double Euclidean(double[] x, double[] y){
    	
    	double val = 0.0;
    	for(int i=0;i<x.length;i++){
    		val = val + (x[i]-y[i])*(x[i]-y[i]);
    	}   
    	double dist = Math.sqrt(val);
    	return dist;
    }    
    
    /**
     * Search the eplison-range Neighborhood objects
     * @param double e , double p, double[][] data
     * @return ArrayList eplison-NN
     */    
    public ArrayList<Integer> eNN(double e, double[] p, double[][] data){
    	
    	ArrayList<Integer> NN = new ArrayList<Integer>();
    	for(int i=0;i<num;i++){
    		if(Euclidean(p,data[i]) < e){
    			NN.add(i);
    		}
    	}	 	
    	return NN;
    }    

    /**
     * Find the local orientation
     * @param ArrayList enn, double[][] data
     * @return ArrayList eigenvector & eigenvalue
     */       
    public ArrayList<double[][]> findLocalOrientation(ArrayList<Integer> enn, double[][] data){
    	
    	ArrayList<double[][]> al = new ArrayList<double[][]>();
    	int len = enn.size();
    	//double[][] conv = new double[len][len];
    	double[][] selData = new double[dim][len];
    	double[] mean = new double[dim];
    	
    	for(int i=0;i<dim;i++){
    		for(int j=0;j<len;j++){
    			selData[i][j] = data[enn.get(j)][i];
    		}
    	}
    	
		//Calculate the mean    	
    	for(int i=0;i<dim;i++){
    		for(int j=0;j<len;j++){
    			mean[i] += selData[i][j];
    		}
    		mean[i]/=len;
    	}
    	
		//Calculate the deviations from the mean
    	double[][] B = new double[dim][len];
		for(int i=0;i<dim;i++){
			for(int j=0;j<len;j++){
				B[i][j] = selData[i][j] - mean[i];  
			}
		}    

		//saveData(B, "B.txt");
		
		Matrix mt = new Matrix(B);
		Matrix conv = mt.times(mt.transpose()).times(1.0/len);		
    	
		//double[][] temp = conv.getArrayCopy();
		//saveData(temp, "conv.txt");
		
        EigenvalueDecomposition eig = new EigenvalueDecomposition(conv);   
        Matrix vectors = eig.getV();
        Matrix values = eig.getD();  
        
        double[][] vectorArray = vectors.getArrayCopy();
        double[][] valueArray = values.getArrayCopy();
        
        //saveData(vectorArray, "vector.txt");
        //saveData(valueArray, "value.txt");
        
        al.add(vectorArray);
        al.add(valueArray);
        
    	return al;
    }
    
    /**
     * Object Mutual Interaction
     * @param ArrayList enn, double[][] data
     * @return ArrayList eigenvector & eigenvalue
     */       
    public void CoInteraction(ArrayList<Integer> enn, int idx, double[][] data, ArrayList<double[][]> orientation){    

		double[][] vector = orientation.get(0);
		double[][] weight = orientation.get(1);
		int len = enn.size();
		
		double sum = 0.0;
		for(int d=0;d<dim;d++){
			sum +=Math.abs(weight[d][d]);
		}
		for(int d=0;d<dim;d++){
			weight[d][d] = Math.abs(weight[d][d])/sum;
			//System.out.println(weight[d][d]);
		}
		
    	for(int i=0;i<len;i++){
    		double[] diff = new double[dim];
    		
    		/*== Project the objects mutual interaction (difference) into the new space by PCA==*/
    		
    		// 1. Pair-wise difference in original space
    		for(int d=0;d<dim;d++){
    			diff[d] = data[enn.get(i)][d] - data[idx][d];
    		}

    		// 2. Calculate the inner product between difference vector and each of the new space axis	
    		for(int k=0;k<dim;k++){
    			double inp =0.0;
	    		for(int d=0;d<dim;d++){
	    			inp += diff[d]*vector[d][k];
	    		}
	    		//System.out.println(Math.signum(diff[0])*inp);
	    		for(int d=0;d<dim;d++){
	    			//nsrc[idx][d] += (weight[d][d])* Math.sin(inp*vector[d][k]);
	    			nsrc[idx][d] += (weight[d][d])* Math.sin(Math.abs(inp))*Math.signum(diff[d]);
	    		}	    		
    		}		
    	}
    	for(int i=0;i<len;i++){
			for(int d=0;d<dim;d++){
				nsrc[idx][d] /=len;
			}
    	}
    }
    
    public void interaction(){
    	
    	long   start2   =   System.currentTimeMillis(); 
    	System.out.println("Number of Objects:" + num + "; Dimensionality: " + dim);
    	boolean loop = true;
    	int loopNum = 0;

    	saveData(nsrc, "synC0.txt"); // Record the initial state of objects
    	double[][] cur = new double[num][dim];
    	
    	// Keep the current values to variable "cur"
    	for(int i=0;i<num;i++){   		
    		for(int j=0;j<dim;j++){
    			cur[i][j] = nsrc[i][j];
    		}	
    	} 
    	
    	// Start to cluster by mutual interaction according to the local structure of objects
    	System.out.println("Start to clustering by local mutual interaction.");
    	while(loop){
    		
    		//System.out.println("Loop:" + loopNum);
    		loopNum = loopNum + 1;
    		
	    	for(int i=0;i<num;i++){	    		
	    		
    			//Search the eplison-neighborhood objects of one object 
    			ArrayList<Integer> enn = eNN(thr, cur[i], cur); 
    			//System.out.println(enn.size());
    			
    			// find the local orientation around the current object
    			if(enn.size()>2){
    				ArrayList<double[][]> orientation = findLocalOrientation(enn, cur);
    				CoInteraction(enn,i, cur, orientation);
    			}
	    	}
			
			//saveData(nsrc, "synC"+String.valueOf(loopNum)+".txt");

			if( loopNum >100){
				loop = false;
			}    	
    	}
    	saveData(nsrc, "syc.txt");
        long   end2   =   System.currentTimeMillis(); 
    	System.out.println("Find the synchronized Phases:"+Long.toString((end2-start2)/1000)+" s.");	
    	
    }
    
    /** Find the related objects in corresponding subspace
     * 
     * @param ArrayList ss,  all possible subspaces 
     * @param ArrayList al,  the all possible spaces of current object
     */
	public ArrayList DBSearch(ArrayList cur){
		
		ArrayList relDB = new ArrayList();
		int id = (Integer)cur.get(0);
		ArrayList al = (ArrayList)cur.get(1); //subspace
		ArrayList ap = (ArrayList)cur.get(2); //synchronized phases
//		for(int i=0;i<al.size();i++){
//			System.out.print(" " + (Integer)al.get(i));
//		}
//		System.out.println("");
//		for(int i=0;i<ap.size();i++){
//			System.out.print(" " + (Double)ap.get(i));
//		}
		relDB.add(al); // Subspace
		relDB.add(ap); // Phases
		ArrayList cluster = new ArrayList(); 
		cluster.add(id); // add the first object to the corresponding cluster
		
		for(int i=0;i<num;i++){
			int flag = 0;
			ArrayList each = (ArrayList)ss.get(i);
			// if this object exist any relevant dimension
			if(each.size()>1){
				ArrayList es = (ArrayList)each.get(1); //get the corresponding subspace
				ArrayList ep = (ArrayList)each.get(2); // the corresponding phases
//				System.out.println("");
//				for(int x=0;x<al.size();x++){
//					System.out.print(" " + (Integer)es.get(x));
//				}
//				System.out.println("");
//				for(int y=0;y<ap.size();y++){
//					System.out.print(" " + (Double)ep.get(y));
//				}
//				
//				System.out.println("es.size " + es.size());
//				System.out.println("al.size " + al.size());
				if(i!=id && es.size()>=al.size()){
					for(int j=0;j<al.size();j++){
						for(int k=0;k<es.size();k++){
							
							//check whether exist any object has same phase in	same subspace						
							if((Integer)es.get(k)==(Integer)al.get(j)){
								//System.out.println("loop: j = " + j + " k =" + k);
								//System.out.println("ep = " + (Double)ep.get(k) + " ap =" + (Double)ap.get(k));
								//System.out.println("ep: "+(Double)ep.get(k));
								//System.out.println("ap: "+(Double)ap.get(j));
									if(((String)ep.get(k)).equals((String)ap.get(j))){
										flag = flag + 1;
									}
							}
						}
						
					}	
				}
				//if the object match, add the object in the cluster
				//System.out.println("size" + al.size()+ " flag=" + flag);
				if(flag==al.size()){
					//System.out.println("sss");
					cluster.add(i);
				}
			}
		}
		relDB.add(cluster); //add the cluster
		relDB.add(cluster.size()); // add the number of the cluster
		return relDB;
	}
   
    /** Search all possible subspace
     * 
     * @param none
     */
	public void subspaceSearch(){
      
		long   start1   =   System.currentTimeMillis(); 
		NumberFormat famatter = NumberFormat.getNumberInstance();  
		famatter.setMaximumFractionDigits(5);
		sdata = new String[num][dim];
		for(int i=0;i<num;i++){                 
			for(int j=0;j<dim;j++){
	            String sData  = famatter.format(nsrc[i][j]);  
	            sdata[i][j] = sData;
			}  
		}
		long   end1   =   System.currentTimeMillis(); 
		System.out.println("Format:"+Long.toString((end1-start1)/1000)+" s.");      
		saveStrrData(sdata,"sdata.txt");
	    
//		for(int i=1;i<num;i++){
//			for(int d=0;d<dim;d++){		
//			System.out.print(" " + fdata[i][d]);	
//			}
//			System.out.println("");
//		}
	    // find the distinct phase at each dimension
		long   start2   =   System.currentTimeMillis(); 
		
		ArrayList al = new ArrayList();
		ArrayList count = new ArrayList();
		for(int d=0;d<dim;d++){
			double phase = 0.0;
			ArrayList temp1 = new ArrayList();
			temp1.add(sdata[0][d]);
			ArrayList temp2 = new ArrayList();
			temp2.add(1);
			for(int i=1;i<num;i++){
				int f = 0;
				//System.out.println("fdata[" + i +"]["+d+"]" + fdata[i][d]);
				for(int j=0;j<temp1.size();j++){
					if(sdata[i][d].equals((String)temp1.get(j))){
						f = 1;
						int curr = (Integer)temp2.get(j);
						curr = curr + 1;
						temp2.set(j, curr);
					}
				}
				if(f==0){
					temp1.add(sdata[i][d]);
					//System.out.println(fdata[i][d]);
					temp2.add(1);
				}
			}
			al.add(temp1);
			count.add(temp2);
		}
	  
		saveStrData(al, "phases.txt");
		saveIntData(count, "count.txt");
		
		ArrayList selPhase = new ArrayList();
		for(int i=0;i<count.size();i++){
			ArrayList temp1 = (ArrayList)count.get(i);
			ArrayList temp2 = (ArrayList)al.get(i);
			ArrayList temp3 = new ArrayList();
			for(int j=0;j<temp1.size();j++){
				if((Integer)temp1.get(j)>10){
					temp3.add((String)temp2.get(j));
				}
			}
			selPhase.add(temp3);
		}
		this.saveStrData(selPhase, "selPhase.txt");
		
	    long   end2   =   System.currentTimeMillis(); 
		System.out.println("Find the synchronized Phases:"+Long.toString((end2-start2)/1000)+" s.");	
		
		// Find the possible all relevant dimensions for each point
		ss = new ArrayList();
		for(int i=0;i<num;i++){
			ArrayList eachRelDim = new ArrayList();
			ArrayList relDim = new ArrayList();
			ArrayList relPhase = new ArrayList();
			for(int d=0;d<dim;d++){
				for(int k=0;k<((ArrayList)selPhase.get(d)).size();k++){
					if(sdata[i][d].equals((String)((ArrayList)selPhase.get(d)).get(k))){						
						relDim.add(d); // Relevant dimension	
						relPhase.add(sdata[i][d]); //The corresponding phase value
					}
				}
			}
			eachRelDim.add(i); //Object ID
			if(relDim.size()>0){
				eachRelDim.add(relDim); // Relevant Dimensions
				eachRelDim.add(relPhase); //The corresponding phase values
			}
			ss.add(eachRelDim);
		}	
			
		// find the related objects in corresponding subspace
		ArrayList DB = new ArrayList(); //Save the all relevant information, including subspace, phases, cluster and number of cluster
		ArrayList rsc = new ArrayList(); // Save the real subspace
		ArrayList psc = new ArrayList(); // for save each object's subspace
		for(int i=0;i<num;i++){
			// For each candidate subspace
			ArrayList cur = (ArrayList)ss.get(i); // the information for each Object, including object id, relevant dimensions and phase values 
			// If there exist any relevant dimension for the object
			if(cur.size()>1){ 
				//System.out.println("s=" + i);
				//first subspace search
				if(rsc.size()==0){
					ArrayList sc = DBSearch(cur);
					if((Integer)sc.get(sc.size()-1)>10){
						DB.add(sc);
						rsc.add((ArrayList)cur.get(1));
						psc.add((ArrayList)cur.get(2));
					}
				}else{
					
					ArrayList as = (ArrayList)cur.get(1);
					ArrayList ap = (ArrayList)cur.get(2);
					int f = 0;
					
					for(int k=0;k<rsc.size();k++){
						if(as.size()==((ArrayList)rsc.get(k)).size()){
							ArrayList rap = (ArrayList)psc.get(k);
							for(int p=0;p<as.size();p++){
								if(((String)rap.get(p)).equals((String)ap.get(p))){
									f = f + 1;
								}
							}
						}
					}
					
					// a new subspace
					if(f!=as.size()){
						
						System.out.println("new space... f=" + f + " as =" + as.size());
						ArrayList sc = DBSearch(cur);
						if((Integer)sc.get(sc.size()-1)>10){
							//System.out.println("test....");
							DB.add(sc);
							rsc.add((ArrayList)cur.get(1));
							psc.add((ArrayList)cur.get(2));
						}
					}
				}
				
			}			
		}

		System.out.println("Totol number of objects: "+num);
		System.out.println("Totol number of subspace: "+rsc.size());		
		for(int i=0;i<DB.size();i++){
			ArrayList res = (ArrayList)DB.get(i);
			System.out.print("subspace " + i +": [");
			ArrayList subs = (ArrayList)res.get(0);
			ArrayList phs = (ArrayList)res.get(1);
			String fn = "cluster";
			for(int j=0;j<subs.size();j++){
				System.out.print(""+subs.get(j)+" ");
				fn = fn+"_"+subs.get(j);
			}
			fn = fn + "_" +i+ ".txt";
			System.out.print("]");
			saveCluster((ArrayList)res.get(2),fn);
			System.out.print("  Synchronized Phases: [");
			for(int j=0;j<phs.size();j++){
				System.out.print(""+phs.get(j)+" ");
			}
			System.out.print("]");
			
			System.out.println(" "+(Integer)res.get(3));
		}		
	}
 
	 /**
     * main function 
     */
    public static void main(String[] args) {
    	
    	long   start   =   System.currentTimeMillis(); 
    	System.out.println("test=" + ("-0".equals("-0")));
    	subTest cl = new subTest("lineInPlane3d.txt",0.05); 
    	cl.loadData();
    	cl.norm();      	
    	cl.interaction(); 
    	cl.subspaceSearch();
    	
  	  	long   end   =   System.currentTimeMillis(); 
  	  	System.out.println("All running time:"+Long.toString((end-start)/1000)+" s.");      
  	  	System.out.println("Ending");	
    }		
}

